<?php
/**
 * Class for checking the strength of a password.
 *
 * @package components
 * @subpackage password_strength_check
 * @version $Id: password_strength_check.class.php 5892 2012-10-31 11:05:29Z avel $
 * @license http://opensource.org/licenses/gpl-license.php GNU Public License
 * @author Kostas Koukopoulos <kouk@noc.uoa.gr>
 * @author Alexandros Vellis <avel@noc.uoa.gr>
 */

/**
 * The checkpassword class performs a series of strength checks on a password
 * string.
 */
class passwordStrengthCheck extends passwordStrengthCheckCollection
{
    /**
     * @var array Active regural expression checks for passwords.
     */
    private $_pwd_regexps = array(); 
    private $_pwd_regexps_counts = array(); 

    /**
     * @var array Active regural expression checks for username.
     */
    private $_uid_regexps = array();
    
    private $_messages = array();

    private $PW_CHECK_LEVENSHTEIN = 2;
    private $PW_CHECK_MIN_LEN = 6;
    private $PW_CHECK_MIN_UNIQ = 5;
    private $PW_CHECK_MIN_LCS = 40;
    private $PW_CHECK_MIN_NON_ALPHA = 2;
    private $PW_MIN_CONSECUTIVE_NUMBERS = 3;

    /**
     * Constructor function that initializes the environment & arguments for the tests.
     * @param $config array Injected configuration
     */
    function __construct($config)
    {
        // Set up Regexps for regex check & respective messages.
        $prev_text_domain = textdomain(null);
        textdomain('password_strength_check');
        
        $arr = array('PW_CHECK_LEVENSHTEIN','PW_CHECK_MIN_LEN','PW_CHECK_MIN_UNIQ','PW_CHECK_MIN_LCS', 'PW_CHECK_MIN_NON_ALPHA');
        foreach($arr as $a) {
            if(isset($config[$a])) {
                $this->$a = $config[$a];
            } else {
                $this->$a = false;
            }
        }

        $this->_pwd_regexps['/[^0-9]/'] = dgettext('password_strength_check', "The password must not consist only of numbers");

        if($this->PW_CHECK_MIN_NON_ALPHA > 1) {
            
            $this->_pwd_regexps_counts['/[^A-z]/'] = sprintf(
                dgettext('password_strength_check', "At least %s non-alpha characters (numbers or symbols), must appear in password"),
                $this->PW_CHECK_MIN_NON_ALPHA
            );

        } elseif($this->PW_CHECK_MIN_NON_ALPHA == 1){
            $this->_pwd_regexps['/[^A-z]/'] = dgettext('password_strength_check', "At least one non-alpha character, i.e. number or symbol, must appear in password");
            
        }


        // Set up informational & error messages for tests
        // an entry in the messages array has these values:
        //  index 0 => The informational message (for displaying rules)
        //  index 1 => the error message, for reporting additional information or hints on how
        //             to fix the problem. (This can be empty and set later for different messages).

        // for regexp test this message changes a bit depending on the configuration
        if($this->PW_CHECK_MIN_NON_ALPHA > 1) {
            $this->_messages['regexp'] = array(
                sprintf( _("At least %s non-alpha characters (numbers or symbols), must appear in password"), $this->PW_CHECK_MIN_NON_ALPHA),
                sprintf( _("At least %s non-alpha characters (numbers or symbols), must appear in password"), $this->PW_CHECK_MIN_NON_ALPHA)
            );

        } elseif($this->PW_CHECK_MIN_NON_ALPHA == 1){
            $this->_messages['regexp'] = array(
                _("Password must contain a number or symbol."),
                ''
            );
        }

        $this->_messages['length'] = array(
            sprintf( _("Password must be at least %s characters long."), $this->PW_CHECK_MIN_LEN) ,
            sprintf( _("Password must be at least %s characters long."), $this->PW_CHECK_MIN_LEN)
        );
        $this->_messages['unique'] = array(
            sprintf( _("Password must have at least %s different characters."), $this->PW_CHECK_MIN_UNIQ) ,
            sprintf( _("Password must have at least %s different characters."), $this->PW_CHECK_MIN_UNIQ)
        );
        $this->_messages['similarity'] = array(
            _("Password should not be very similar to the username."),
            ''
        );
        $this->_messages['consecutivenumbers'] = array(
            _("Password should not contain more than 3 ascending or descending numbers in succession") ,
            _("Password should not contain more than 3 ascending or descending numbers in succession")
        );
        
        if($prev_text_domain) textdomain($prev_text_domain);
    }

    /**
     * Regexp Check.
     *
     * @param array $data
     * @return mixed An error message if there was a failure, or void on
     * success.
     */
    function regexpTest($data) {
        foreach ($this->_pwd_regexps as $exp => $msg) {
            if( preg_match ($exp, $data[1]) == FALSE)
                return $msg;
        }

        foreach ($this->_pwd_regexps_counts as $exp => $msg) {
            $ret = preg_match_all($exp, $data[1]);
            if($ret < $this->PW_CHECK_MIN_NON_ALPHA) {
                return $msg;
            }
        }

        foreach ($this->_uid_regexps as $exp => $msg) {
            if( preg_match ($exp, $data[0]) == FALSE) 
                return $msg;
        }
    }

    /**
     * Length Test
     *
     * @param array $data
     * @return mixed An error message if there was a failure, or void on
     * success.
     */
    function lengthTest($data) {
        if ($this->PW_CHECK_MIN_LEN and strlen($data[1]) < $this->PW_CHECK_MIN_LEN) {
            return $this->_messages['length'][1];
        }
    }

    /**
     * Uniqueness test. Password should have at least some different
     * characters.
     *
     * @param array $data
     * @return mixed An error message if there was a failure, or void on
     * success.
     */
    function uniqueTest($data) {
        if ($this->PW_CHECK_MIN_UNIQ and strlen (count_chars ($data[1], 3)) < $this->PW_CHECK_MIN_UNIQ) {
            return $this->_messages['unique'][1];
        }
    }

    /**
     * Similarity with username.
     *
     * @param array $data
     * @return mixed An error message if there was a failure, or void on
     * success.
     */
    function similarityTest($data) {
        if ($this->PW_CHECK_LEVENSHTEIN and levenshtein($data[0], $data[1]) < $this->PW_CHECK_LEVENSHTEIN) {
            return dgettext('password_strength_check', "Password should not be too much like the username.");
        }

        // Additional similarity with username test (Uses LCS functions).
        if ($this->PW_CHECK_MIN_LCS and get_lcs($data[0], $data[1]) >= $this->PW_CHECK_MIN_LCS) {
            return sprintf( dgettext('password_strength_check', "Password should not contain more than %s percent of Username."),
                $this->PW_CHECK_MIN_LCS);
        }
    }

    /**
     * Check if password has n consecutive numbers that are ascending or descending.
     *
     * @param array $data
     * @return mixed An error message if there was a failure, or void on
     * success.
     */
    function consecutivenumbersTest($data)
    {
        if(!$this->PW_MIN_CONSECUTIVE_NUMBERS) {
            return;
        }

        $flag = 0;
        $prev = false;
        $sign = 0;
        
        for($i=0; $i<strlen((string)($data[1])); $i++) {
            $cur = $i;
            if(is_numeric($data[1][$i])) {
                if($flag && $prev !== false) {
                    // print " comparing ".$data[1][$i]. " and $prev";
                    if(!$sign) {
                        if(abs((int)$data[1][$i] - (int)$prev) == 1 ){
                            $sign = (int)$data[1][$i] - (int)$prev;
                        }
                    }
                    if($sign && ((int)$data[1][$i] - (int)$prev == $sign)) {
                        $flag++;
                        if($flag >= $this->PW_MIN_CONSECUTIVE_NUMBERS) {
                            return sprintf($this->_messages['consecutivenumbers'][1], $flag);
                        }
                    } else {
                        // reset + go one character backwards
                        $flag = 0;
                        $sign = 0;
                        $i--;
                    }
                } else {
                    $flag++;
                }
                $prev = $data[1][$cur];
            } else {
                $flag = 0;
                $sign = 0;
                $prev = false;
            }
        }
    }
}

